/****************************************************************************
 *  Familia Lignum - Genealogical program                                   *
 *  Copyright (C) 2011-2012 Rafał Długołęcki <rafal@dlugolecki.net.pl>      *
 *                                                                          *
 *  This program is free software; you can redistribute it and/or modify    *
 *  it under the terms of the GNU General Public License as published by    *
 *  the Free Software Foundation; version 2 of the License.                 *
 *                                                                          *
 *  This program is distributed in the hope that it will be useful,         *
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of          *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           *
 *  GNU General Public License for more details.                            *
 *                                                                          *
 *  You should have received a copy of the GNU General Public License along *
 *  with this program; if not, write to the Free Software Foundation, Inc., *
 *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.             *
 *                                                                          *
 ****************************************************************************/

#include <iostream>
#include <gtkmm.h>
#include <glibmm/i18n.h>
#include <sigc++/sigc++.h>

#include "../config.h"

#include "window.h"
#include "file/file.h"
#include "file/gno/gno.h"

Window::Window() : box(Gtk::ORIENTATION_VERTICAL) {
  set_title(PACKAGE_STRING);
  set_default_size(400, 400);
  add(box);

  m_refActionGroup = Gtk::ActionGroup::create();
  m_refActionGroup->add(Gtk::Action::create("FileMenu", Gtk::Stock::FILE));
  m_refActionGroup->add(Gtk::Action::create("FileOpen", Gtk::Stock::OPEN),
			sigc::mem_fun(*this, &Window::on_menu_file_open));
  m_refActionGroup->add(Gtk::Action::create("FileQuit", Gtk::Stock::QUIT),
			sigc::mem_fun(*this, &Window::on_menu_file_quit));


  m_refUIManager = Gtk::UIManager::create();
  m_refUIManager->insert_action_group(m_refActionGroup);

  add_accel_group(m_refUIManager->get_accel_group());

  //Layout the actions in a menubar and toolbar:
  //        "      <menuitem action='FileNew'/>"
  Glib::ustring ui_info = 
        "<ui>"
        "  <menubar name='MenuBar'>"
        "    <menu action='FileMenu'>"
        "      <menuitem action='FileOpen'/>"
        "      <separator/>"
        "      <menuitem action='FileQuit'/>"
        "    </menu>"
        "  </menubar>"
    "</ui>";

  try {
    m_refUIManager->add_ui_from_string(ui_info);
  }
  catch(const Glib::Error& ex) {
    std::cerr << "building menus failed: " <<  ex.what();
  }

  //Get the menubar and toolbar widgets, and add them to a container
  //widget:
  Gtk::Widget* pMenubar = m_refUIManager->get_widget("/MenuBar");
  if(pMenubar)
    box.pack_start(*pMenubar, Gtk::PACK_SHRINK);

  tabs.signal_page_added().connect(
        sigc::mem_fun(*this, &Window::on_notebook_page_added));
  tabs.signal_page_removed().connect(
        sigc::mem_fun(*this, &Window::on_notebook_page_removed));

  box.pack_start(tabs, Gtk::PACK_EXPAND_WIDGET);

  show_all_children();
}

Window::~Window() {
}

void Window::on_menu_file_new() {

}

void Window::on_menu_file_open() {
  Gtk::FileChooserDialog dialog(_("Please choose a file"), Gtk::FILE_CHOOSER_ACTION_OPEN);
  dialog.set_transient_for(*this);
  dialog.add_button(Gtk::Stock::CANCEL, Gtk::RESPONSE_CANCEL);
  dialog.add_button(Gtk::Stock::OPEN, Gtk::RESPONSE_OK);

  Glib::RefPtr<Gtk::FileFilter> filter_gno = Gtk::FileFilter::create();
  filter_gno->set_name("Gno Files");
  filter_gno->add_pattern("*.gno");
  dialog.add_filter(filter_gno);

  Glib::RefPtr<Gtk::FileFilter> filter_any = Gtk::FileFilter::create();
  filter_any->set_name("Any files");
  filter_any->add_pattern("*");
  dialog.add_filter(filter_any);


  //Show the dialog and wait for a user response:
  int result = dialog.run();

  //Handle the response:
  switch(result) {
    case(Gtk::RESPONSE_OK):
      {
	String filepath = dialog.get_filename();
	String filename = filepath.substr(filepath.find_last_of("/") + 1);
	String extension = filename.substr(filename.find_last_of(".") + 1);
	file::File * file = NULL;

	if(extension == "gno") {
	  file = new file::Gno(filepath);
	}

	if(file)
	  tabs.append_page(*file, filename);

	tabs.show_all();
	break;
      }
    case(Gtk::RESPONSE_CANCEL):
      {
	//	std::cout << "Cancel clicked." << std::endl;
	break;
      }
    default:
      {
	std::cerr << "Unexpected button clicked in opendialog." << std::endl;
	break;
      }
    }
}

void Window::on_menu_file_quit() {
  hide(); //Closes the main window to stop the Gtk::Main::run().
}

void Window::on_notebook_page_added(Widget* page, guint page_num) {
  std::cout << "Page added " << page->get_name() << " " << page_num << std::endl;
  /*
  file::File * f = dynamic_cast<file::File*>(page);

  if(f->details.extension == "gno") {
    std::cout << "This is a gno file." << std::endl;
  }
  */
}

void Window::on_notebook_page_removed(Widget* page, guint page_num) {
  std::cout << "Page removed " << page->get_name() << " " << page_num << std::endl;
}
